unit TTSU;

//{$define MOUTH_IMAGES_ARE_PRESENT}

interface

uses
  ComObj, Menus, ActiveX,
{$ifdef CONDITIONALEXPRESSIONS}
  Variants,
{$endif}
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, OleServer, SpeechLib_TLB, ExtCtrls, ImgList;

type
  TForm1 = class(TForm)
    SpVoice: TSpVoice;
    Button1: TButton;
    Memo1: TMemo;
    SpFileStream: TSpFileStream;
    Button2: TButton;
    Button3: TButton;
    ListBox1: TListBox;
    ListBox2: TListBox;
    Memo2: TMemo;
    ImageList1: TImageList;
    PaintBox1: TPaintBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure ListBox1Click(Sender: TObject);
    procedure SpVoicePhoneme(Sender: TObject; StreamNumber: Integer;
      StreamPosition: OleVariant; Duration: Integer; NextPhoneId: Smallint;
      Feature: TOleEnum; CurrentPhoneId: Smallint);
    procedure SpVoiceWord(Sender: TObject; StreamNumber: Integer;
      StreamPosition: OleVariant; CharacterPosition, Length: Integer);
    procedure SpVoiceViseme(Sender: TObject; StreamNumber: Integer;
      StreamPosition: OleVariant; Duration: Integer; NextVisemeId, Feature,
      CurrentVisemeId: TOleEnum);
    procedure PaintBox1Paint(Sender: TObject);
  private
    procedure MsgBoxButtonClick(Sender: TObject);
    procedure ReadVCLDialog(Form: TCustomForm);
    procedure ScreenFormChange(Sender: TObject);
{$ifdef MOUTH_IMAGES_ARE_PRESENT}
    procedure SetupMouthImages;
{$endif}
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.Button1Click(Sender: TObject);
begin
  //Synchronous speaking (the voice events will NOT fire)
  //SpVoice.Speak(Memo1.Text, SVSFDefault);

  //Asynchronous speaking (the voice events will fire)
  SpVoice.Speak(Memo1.Text, SVSFlagsAsync);

  //Speak synchronously, with XML tags to control the pitch/volume/speed/etc.
  //SpVoice.Speak(Memo1.Text, SVSFIsXML);

  //N.B. you can use XML tags in asynchronous speech using the
  //SVSFlagsAsync flag, however you must ensure that the very first,
  //part of the text is an XML tag (even an empty one will do)


  //"Speak" the contents of a WAV file
  //SpVoice.Speak('Some WAV file.wav', SVSFIsFilename);

  //Record TTS speech in a WAV file
  //SpFileStream.Open('Some WAV file.wav', SSFMCreateForWrite, False);
  //SpVoice.AudioOutputStream := SpFileStream.DefaultInterface;
  //SpVoice.Speak(Memo1.Text, SVSFDefault);
  //SpFileStream.Close;

  Memo1.SetFocus
end;

procedure TForm1.Button2Click(Sender: TObject);
const
// Constants for enum WdUnits
  wdCharacter = $00000001;
  wdParagraph = $00000004;
// Constants for enum WdMovementType
  wdExtend = $00000001;
var
  Moved: Integer;
  Txt: String;
  MSWord: Variant;
const
  WordDocToRead = 'Fill in a filename here';
begin
  MSWord := CreateOleObject('Word.Application');
  MSWord.Documents.Open(FileName := WordDocToRead);
  (Sender as TButton).Enabled := False;
  Moved := 2;
  while Moved > 1 do
  begin
    //Select next paragraph
    Moved := MSWord.Selection.EndOf(Unit := wdParagraph, Extend := wdExtend);
    if Moved > 1 then
    begin
      Txt := Trim(MSWord.Selection.Text);
      if Length(Txt) > 0 then
        SpVoice.Speak(Txt, SVSFlagsAsync);
      Application.ProcessMessages;
      //Move to start of next paragraph
      MSWord.Selection.MoveRight(Unit := wdCharacter);
    end
  end;
  MSWord.Quit;
  MSWord := Unassigned;
  TButton(Sender).Enabled := True;
end;

var
  Voice: Variant;

procedure TForm1.FormCreate(Sender: TObject);
var
  I: Integer;
  SOTokens: ISpeechObjectTokens;
begin
  Screen.OnActiveFormChange := ScreenFormChange;
  SOTokens := SpVoice.GetVoices('', '');
  for I := 0 to SOTokens.Count - 1 do
    ListBox1.Items.Add(SOTokens.Item(I).GetDescription(0));
  ListBox1.ItemIndex := ListBox1.Items.IndexOf(SpVoice.Voice.GetDescription(0));
  ListBox1.OnClick(ListBox1);
{$ifdef MOUTH_IMAGES_ARE_PRESENT}
  SetupMouthImages
{$endif}
end;

procedure TForm1.ScreenFormChange(Sender: TObject);
begin
  if Assigned(Screen.ActiveForm) and
     (Screen.ActiveForm.ClassName = 'TMessageForm') then
    ReadVCLDialog(Screen.ActiveForm)
end;

const
  SVSFlagsAsync = 1;
{$ifndef CONDITIONALEXPRESSIONS}
  sLineBreak = #13#10;
{$endif}

procedure TForm1.ReadVCLDialog(Form: TCustomForm);
var
  I: Integer;
  ButtonCaptions, LabelCaption, DialogText: string;
begin
  try
    if VarType(Voice) <> varDispatch then
      Voice := CreateOleObject('SAPI.SpVoice');
    for I := 0 to Form.ComponentCount - 1 do
      if Form.Components[I] is TLabel then
        LabelCaption := TLabel(Form.Components[I]).Caption
      else
        if Form.Components[I] is TButton then
        begin
          ButtonCaptions := Format('%s%s, ',
            [ButtonCaptions, TButton(Form.Components[I]).Caption]);
          TButton(Form.Components[I]).OnClick := MsgBoxButtonClick;
        end;
    ButtonCaptions := StringReplace(ButtonCaptions,'&','', [rfReplaceAll]);
    DialogText := Format('%s.%s%s.%s%s',
      [Form.Caption, sLineBreak, LabelCaption, sLineBreak, ButtonCaptions]);
    Memo1.Text := DialogText;
    Voice.Speak(DialogText, SVSFlagsAsync)
  except
    //pretend everything is okay
  end
end;

procedure TForm1.MsgBoxButtonClick(Sender: TObject);
begin
  Voice.Skip('Sentence', Maxint);
  Voice.Speak(
    StripHotkey((Sender as TButton).Caption),
    SVSFlagsAsync)
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
  ShowMessage('Hello');
  MessageDlg('Hello', mtConfirmation, mbYesNoCancel, 0);
end;

procedure TForm1.ListBox1Click(Sender: TObject);
begin
  SpVoice.Voice := SpVoice.GetVoices('', '').Item(ListBox1.ItemIndex);
  ListBox2.Clear;
  ListBox2.Items.Add(Format('Name: %s', [SpVoice.Voice.GetAttribute('Name')]));
  ListBox2.Items.Add(Format('Vendor: %s', [SpVoice.Voice.GetAttribute('Vendor')]));
  ListBox2.Items.Add(Format('Age: %s', [SpVoice.Voice.GetAttribute('Age')]));
  ListBox2.Items.Add(Format('Gender: %s', [SpVoice.Voice.GetAttribute('Gender')]));
{$ifdef MOUTH_IMAGES_ARE_PRESENT}
  SetupMouthImages
{$endif}
  ListBox2.Items.Add(Format('Language: %s', [SpVoice.Voice.GetAttribute('Language')]));
  ListBox2.Items.Add(Format('Reg key: %s', [SpVoice.Voice.Id]));
end;

procedure TForm1.SpVoicePhoneme(Sender: TObject; StreamNumber: Integer;
  StreamPosition: OleVariant; Duration: Integer; NextPhoneId: Smallint;
  Feature: TOleEnum; CurrentPhoneId: Smallint);
const
  //See the SAPI 5.1 SDK help file for phoneme definitions:
  //  White Papers, TTS Engine Vendor Porting Guide, Appendix A
  Phonemes: array[1..49] of String = (
    '-', '!', '&', ',', '.', '?', '_',
    '1', '2', 'aa', 'ae', 'ah', 'ao', 'aw',
    'ax', 'ay', 'b', 'ch', 'd', 'dh', 'eh',
    'er', 'ey', 'f', 'g', 'h', 'ih', 'iy',
    'jh', 'k', 'l', 'm', 'n', 'ng', 'ow',
    'oy', 'p', 'r', 's', 'sh', 't', 'th',
    'uh', 'uw', 'v', 'w', 'y', 'z', 'zh'
  );
begin
  if CurrentPhoneId <> 7 then
    Memo2.Text := Memo2.Text + Phonemes[CurrentPhoneId] + #32
end;

procedure TForm1.SpVoiceWord(Sender: TObject; StreamNumber: Integer;
  StreamPosition: OleVariant; CharacterPosition, Length: Integer);
begin
  Memo1.SelStart := CharacterPosition;
  Memo1.SelLength := Length; //highlight word
end;

{$ifdef MOUTH_IMAGES_ARE_PRESENT}
procedure TForm1.SetupMouthImages;
var
  Bmp: TBitmap;
  I: Integer;
const
  //These directories each need 22 images (Viseme0.bmp to Viseme21.bmp).
  //Viseme0.bmp is the silence viseme
  //The rest are visual representations of various phonemes
  //See the SAPI 5.1 SDK help file for viseme definitions:
  //  White Papers, TTS Engine Vendor Porting Guide, Appendix B

  //When you have the images you should define the MOUTH_IMAGES_ARE_PRESENT
  //symbol to get the mouth image code compiled in
  MaleImagesDir = 'Fill in a directory here';
  FemaleImagesDir = 'Fill in a directory here';
begin
  Bmp := TBitmap.Create;
  try
    ImageList1.Clear;
    ImageList1.Masked := False;
    ImageList1.Width := 160;
    ImageList1.Height := 124;
    if CompareText(SpVoice.Voice.GetAttribute('Gender'), 'Male') = 0 then
      ChDir(MaleImagesDir)
    else
      ChDir(FemaleImagesDir);
    for I := 0 to 21 do
    begin
      Bmp.LoadFromFile(Format('Viseme%d.bmp', [I]));
      ImageList1.AddMasked(Bmp, Bmp.TransparentColor);
    end;
    PaintBox1.Width := Bmp.Width;
    PaintBox1.Height := Bmp.Height;
    PaintBox1.Invalidate
  finally
    Bmp.Free
  end
end;
{$endif}

procedure TForm1.SpVoiceViseme(Sender: TObject; StreamNumber: Integer;
  StreamPosition: OleVariant; Duration: Integer; NextVisemeId, Feature,
  CurrentVisemeId: TOleEnum);
begin
{$ifdef MOUTH_IMAGES_ARE_PRESENT}
  //Draw the appropriate mouth viseme
  ImageList1.Draw(PaintBox1.Canvas, 0, 0, CurrentVisemeId);
{$endif}
end;

procedure TForm1.PaintBox1Paint(Sender: TObject);
begin
{$ifdef MOUTH_IMAGES_ARE_PRESENT}
  ImageList1.Draw(PaintBox1.Canvas, 0, 0, 0);
{$endif}
end;

end.
